/*
  DC2011 Wrokshop 1 Code
  Written By Nick Marchuk  1/26/2011
  Take a look at ReadMe.pdf for more information
*/

#include <plib.h>
#include "NU32v2_DC2011.h"

// L293D pins
#define DIR     LATDbits.LATD1
#define EN      LATDbits.LATD2

void initADC();

int main(void) {
  int i = 2;
  int duty = 0;
  unsigned short int channel0;	// for the ADC read

  SYSTEMConfig(SYS_FREQ, SYS_CFG_ALL);

  // configure for multi-vectored mode
  INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);

  // enable multi-vector interrupts 
  INTEnableSystemMultiVectoredInt(); 

  // initialize serial port and interrupt
  initSerialNU32v2();
    
  initLEDs(); 	// set LED pins as outputs

  // setup direction and enable pins for the h bridge
  TRISDCLR = 0x0006; // make D1 and D2 outputs, to make an input, use TRISDSET = ...

  DIR = 0;
  EN = 0; // start off

  WriteString(UART3, "\r\nHello World!\r\n");

  sprintf(RS232_Out_Buffer, "the value of i is: %d\r\n", i);
  WriteString(UART3, RS232_Out_Buffer);

  // initialize the ADC
  initADC();

  // initialize the PWM
  OpenTimer2(T2_ON | T2_PS_1_4, 1000); // for 20kHz PWM: 80000000Hz / 4ps / 20000Hz = 1000
  OpenOC1(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); // PWM output on Pin D0
  SetDCOC1PWM(duty); // int duty from 0-1000, because of the number used in OpenTimer2

  EN = 1; // turn on the h bridge

  while(1) {
    if (!SW) { // if the switch is pushed
      channel0 = ReadADC10(0);  // read the ADC, 0 to 3.3V converted to 0 to 1023
      if (channel0 > 1000) {  // max out the value at 1000
        channel0 = 1000;
      }
      duty = channel0;
      SetDCOC1PWM(duty);  // make the duty cycle the same as the adc value (duty must be 0-1000)
    }
  }

  return 0;
} // end main

// UART3 interrupt handler, priority level 2
void __ISR(_UART_3_VECTOR, ipl2) IntUart3Handler(void) {
  // Is this an RX interrupt?
  if(INTGetFlag(INT_SOURCE_UART_RX(UART3))){
    char data = UARTGetDataByte(UART3);

    // Echo what we just received
    PutCharacter(UART3, data); 

    // do something with the data
    if(data =='a') {
      LED0 = 0;  LED1 = 1;
      DIR = 0; // change motor direction, but also messes up your speed! See ReadMe.pdf
    }
    else {
      LED0 = 1; LED1 = 0;
      DIR = 1; // change motor direction, but also messes up your speed! See ReadMe.pdf
    }

    // Clear the RX interrupt Flag
    INTClearFlag(INT_SOURCE_UART_RX(UART3)); 
  }

  // We don't care about TX interrupt
  if(INTGetFlag(INT_SOURCE_UART_TX(UART3))) {
    INTClearFlag(INT_SOURCE_UART_TX(UART3));
  }
}

void initADC() {
  // configure and enable the ADC
  CloseADC10();	// ensure the ADC is off before setting the configuration

  // define setup parameters for OpenADC10
  // Turn module on | output in integer | trigger mode auto | enable  autosample
  #define PARAM1  ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON

  // define setup parameters for OpenADC10
  // ADC ref external    | disable offset test    | enable scan mode | perform 2 samples | use one buffer | use MUXA mode
  #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF

  // define setup parameters for OpenADC10
  // 				  use ADC internal clock | set sample time
  #define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15

  // define setup parameters for OpenADC10
  // set AN0 -> pin B0
  #define PARAM4	ENABLE_AN0_ANA

  // define setup parameters for OpenADC10
  // do not assign channels to scan
  #define PARAM5	SKIP_SCAN_AN1 | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

  // use ground as neg ref for A | use AN0 (B0) for input A
  // configure to sample AN0
  SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF); // configure to sample AN0
  OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using parameter define above

  EnableADC10(); // Enable the ADC

  while ( ! mAD1GetIntFlag() ) { } // wait for the first conversion to complete so there will be valid data in ADC result registers
}

